/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * Copyright (c) 2004-2013  HUBzero Foundation, LLC
 *
 *  Authors:
 *    Wei Qiao <qiaow@purdue.edu>
 */
#ifndef NV_CAMERA_H
#define NV_CAMERA_H

#include <vrmath/Matrix4x4d.h>
#include <vrmath/Vector3f.h>

#include "config.h"

namespace nv {

class Camera
{
public:
    enum AxisDirection {
        X_POS = 1,
        Y_POS = 2,
        Z_POS = 3,
        X_NEG = -1,
        Y_NEG = -2,
        Z_NEG = -3
    };

    Camera(int x, int y, int width, int height);

    ~Camera()
    {}

    void setUpdir(AxisDirection dir)
    {
        setUpDirMatrix(dir);
    }

    AxisDirection getUpdir()
    {
        return _updir;
    }

    void setPosition(const vrmath::Vector3f& pos)
    {
        _position = pos;
    }

    const vrmath::Vector3f& getPosition() const
    {
        return _position;
    }

    void x(float loc_x)
    {
        _position.x = loc_x;
    }

    float x() const
    {
        return _position.x;
    }

    void y(float loc_y)
    {
        _position.y = loc_y;
    }

    float y() const
    {
        return _position.y;
    }

    void z(float loc_z)
    {
        _position.z = loc_z;
    }

    float z() const
    {
        return _position.z;
    }

    void orient(double *quat);

    void orient(float angleX, float angleY, float angleZ);

    void setFov(float fov)
    {
        _fov = fov;
    }

    float getFov() const
    {
        return _fov;
    }

    void reset(const vrmath::Vector3f& bboxMin,
               const vrmath::Vector3f& bboxMax,
               bool resetOrientation = false);

    void resetClippingRange(const vrmath::Vector3f& bboxMin,
                            const vrmath::Vector3f& bboxMax);

    void setViewport(int x, int y, int width, int height)
    {
        _viewport[0] = x;
        _viewport[1] = y;
        _viewport[2] = width;
        _viewport[3] = height;
    }

    /**
     * \brief Make the camera setting active, this has to be
     * called before drawing things
     */
    void initialize();

    void print() const;

    void getProjectionMatrix(vrmath::Matrix4x4d& mat);

    const vrmath::Matrix4x4d& getUpDirMatrix() const;

private:
    void setClippingRange(float near, float far)
    {
        _near = near;
        _far = far;
    }

    void setUpDirMatrix(AxisDirection dir);

    AxisDirection _updir;

    /// Position of the camera in the scene
    vrmath::Vector3f _position;

    /// Model transform for z-up scene
    vrmath::Matrix4x4d _updirMatrix;
    /// Camera orientation
    vrmath::Matrix4x4d _rotationMatrix;

    /// Field of view (vertical angle in degrees)
    float _fov;
    /// Near, far z clipping
    float _near, _far;

    // x, y, width, height
    int _viewport[4];
};

}

#endif
